--[[ Requirements for OSC Profiles
For OSC Profiles, there are only three functions to be implemented:
* `controller_info`: as with any other MIDI Device Profile, provides a default assignment list, just as for other control surfaces, and also the pattern recognizing the device (`OSC_pattern`)
* `CSFeedback`: doesn't send any feedback; returns a non-null value for the maximum length of text feedback.
* `CSLabelSize`: returns the maximum length of a text label, if there is one for the assignment.

All OSC communication is done by Logic Pro — you don't have to write it.]]

--------------------------------------------
--- ControlIDs
------------------
--[[
In general the field ControlID seems not to be required for an entry in the Item table, however some entries that seem
similar to others will not show up correctly unless they have a controlID assigned to them (e.g.
'fader-bank-' and fader-bank+' where only the first entry shows up when both have no ControlID assigned to them).

The ControlIDs seem to be arbitrary as long as they are unique.]]

-- Transport
kIdStop = 1
kIdPlay = 2
kIdPause = 3
kIdRecord = 4
kIdForward = 5
kIdRewind = 6
kIdForwardShuttle = 7
kIdRewindShuttle = 8
kIdCycle = 9
kIdTapTempo = 10
kIdQuantize = 11
kIdRestart = 12

-- Control
kIdUndo = 13
kIdRedo = 14
kIdMetronome = 15
kIdCountIn = 16
kIdMoveCycleForward = 17
kIdMoveCycleBackwards = 18

-- Navigation
kIdPrevTrack = 19
kIdNextTrack = 20
kIdPrevObject = 21
kIdNextObject = 22
kIdFaderBankMinus = 23
kIdFaderBankPlus = 24
kIdCurrentTrack = 25
kIdParameter0 = 26
kIdMasterVolume = 27

kIdTrackName1 = 28
kIdTrackNumber1 = 29
kIdSelect1 = 30
kIdMute1 = 31
kIdSolo1 = 32
kIdVolume1 = 33
kIdPan1 = 34
kIdAutomation1 = 35
kIdLevelL1 = 36
kIdLevelR1 = 37
kIdTrackName2 = 38
kIdTrackNumber2 = 39
kIdSelect2 = 40
kIdMute2 = 41
kIdSolo2 = 42
kIdVolume2 = 43
kIdPan2 = 44
kIdAutomation2 = 45
kIdLevelL2 = 46
kIdLevelR2 = 47

kIdTrackName3 = 48
kIdTrackNumber3 = 49
kIdSelect3 = 50
kIdMute3 = 51
kIdSolo3 = 52
kIdVolume3 = 53
kIdPan3 = 54
kIdAutomation3 = 55
kIdLevelL3 = 56
kIdLevelR3 = 57

kIdTrackName4 = 58
kIdTrackNumber4 = 59
kIdSelect4 = 60
kIdMute4 = 61
kIdSolo4 = 62
kIdVolume4 = 63
kIdPan4 = 64
kIdAutomation4 = 65
kIdLevelL4 = 66
kIdLevelR4 = 67

kIdTrackName5 = 68
kIdTrackNumber5 = 69
kIdSelect5 = 70
kIdMute5 = 71
kIdSolo5 = 72
kIdVolume5 = 73
kIdPan5 = 74
kIdAutomation5 = 75
kIdLevelL5 = 76
kIdLevelR5 = 77

kIdTrackName6 = 78
kIdTrackNumber6 = 79
kIdSelect6 = 80
kIdMute6 = 81
kIdSolo6 = 82
kIdVolume6 = 83
kIdPan6 = 84
kIdAutomation6 = 85
kIdLevelL6 = 86
kIdLevelR6 = 87

kIdTrackName7 = 88
kIdTrackNumber7 = 89
kIdSelect7 = 90
kIdMute7 = 91
kIdSolo7 = 92
kIdVolume7 = 93
kIdPan7 = 94
kIdAutomation7 = 95
kIdLevelL7 = 96
kIdLevelR7 = 97

kIdTrackName8 = 98
kIdTrackNumber8 = 99
kIdSelect8 = 100
kIdMute8 = 101
kIdSolo8 = 102
kIdVolume8 = 103
kIdPan8 = 104
kIdAutomation8 = 105
kIdLevelL8 = 106
kIdLevelR8 = 107

--[[ Item Table: Connecting controls with key commands

    For a control to trigger a key command, it has to be connected the field 'keyCmd' with the command id or string.
    For items with objectType 'Button' Logic tries to find a matching key command by comparing the control name.
    If the control name matches (ignoring case) any of the key command strings, the control is mapped accordingly to
    the key command.

    E.g. 'Play' is matched to key command 'play' (id: 3), so the field 'keyCmd' is not required. ]]

controls = {

    --------------------------------------------------------------
    --- Transport
    ----------------------------
    [kIdPlay] = { name = 'play', objectType = 'Button', OSCValueChange = '/mas/transport/play' },
    [kIdStop] = { name = 'stop', objectType = 'Button', OSCValueChange = '/mas/transport/stop' },
    [kIdPause] = { name = 'pause', objectType = 'Button', OSCValueChange = '/mas/transport/pause' },
    [kIdRecord] = { name = 'record', objectType = 'Button', OSCValueChange = '/mas/transport/record' },
    [kIdForward] = { name = 'forward', objectType = 'Button', OSCValueChange = '/mas/transport/forward' },
    [kIdRewind] = { name = 'rewind', objectType = 'Button', OSCValueChange = '/mas/transport/rewind' },
    [kIdCycle] = { name = 'cycle', objectType = 'Button', OSCValueChange = '/mas/transport/cycle' },
    [kIdForwardShuttle] = { name = 'forward_shuttle', keyCmd = 1011, OSCValueChange = '/mas/transport/forward_shuttle' },
    [kIdRewindShuttle] = { name = 'rewind_shuttle', keyCmd = 1010, OSCValueChange = '/mas/transport/rewind_shuttle' },
    [kIdTapTempo] = { name = 'tap_tempo', keyCmd = 532, OSCValueChange = '/mas/transport/tap_tempo' },
    [kIdQuantize] = { name = 'quantize', keyCmd = 110, OSCValueChange = '/mas/transport/quantize' },
    [kIdRestart] = { name = 'restart', keyCmd = 14, OSCValueChange = '/mas/transport/restart' },

    --------------------------------------------------------------
    --- Control
    ----------------------------
    [kIdUndo] = { name = 'undo', keyCmd = 761, OSCValueChange = '/mas/control/undo' },
    [kIdRedo] = { name = 'redo', keyCmd = 796, OSCValueChange = '/mas/control/redo' },
    [kIdMetronome] = { name = 'metronome', keyCmd = 474, OSCValueChange = '/mas/control/metronome' },
    [kIdCountIn] = { name = 'count_in', keyCmd = 1291, OSCValueChange = '/mas/control/count_in' }, -- Toggle count-in
    [kIdMoveCycleForward] = { name = 'move_cycle_forward', keyCmd = 699, OSCValueChange = '/mas/control/move_cycle_forward' }, -- Move Locators Forward by Cycle Length / Loop Forward by Loop Length
    [kIdMoveCycleBackwards] = { name = 'move_cycle_backwards', keyCmd = 700, OSCValueChange = '/mas/control/move_cycle_backwards' }, -- Move Locators Backwards by Cycle Length / Loop Backwards by Loop Length

    --------------------------------------------------------------
    --- Navigation
    ----------------------------
    [kIdPrevTrack] = { name = 'prev_track', keyCmd = 1272, OSCValueChange = '/mas/navigation/prev_track' },
    [kIdNextTrack] = { name = 'next_track', keyCmd = 1273, OSCValueChange = '/mas/navigation/next_track' },
    [kIdPrevObject] = { name = 'prev_object', keyCmd = 82, OSCValueChange = '/mas/navigation/prev_object' }, -- Select Previous Region/Cell on Focused Track
    [kIdNextObject] = { name = 'next_object', keyCmd = 83, OSCValueChange = '/mas/navigation/next_object' }, -- Select Next Region/Cell on Focused Track

    [kIdFaderBankMinus] = { name = 'fader-bank-', controlID = kIdFaderBankMinus, valueMode = kAssignRelative, CSGroupObj = ACS_CURMODEFADERBANK, bankType = ABT_BYBANK, OSCValueChange = '/mas/tracks/fader-bank-' },
    [kIdFaderBankPlus] = { name = 'fader-bank+', controlID = kIdFaderBankPlus, valueMode = kAssignRelative, CSGroupObj = ACS_CURMODEFADERBANK, bankType = ABT_BYBANK, OSCValueChange = '/mas/tracks/fader-bank+' },

    [kIdCurrentTrack] = { name = 'current_track', controlID = kIdCurrentTrack, valueMode = kAssignDirect, CSTrack = true, trackParam = CS_NAME, paramName = '@t#', shortParamName = '@t#', OSCLabel = '/mas/tracks/current_index', OSCValueString = '/mas/tracks/current_name' },
    [kIdParameter0] = { name = 'parameter0', CSTrack = true, trackParam = CS_INSTRUMENTPAR1, shortParamName = '@tp', OSCValueChange = '/mas/tracks/parameter0', OSCTouched = '/mas/tracks/parameter0/z', OSCLabel = '/mas/tracks/parameter0/label', OSCValueString = '/mas/tracks/parameter0/value' },
    [kIdMasterVolume] = { name = 'volume-master', master = 0, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/master/volume', OSCTouched = '/mas/tracks/master/volume/z' },

    --- Track 1
    [kIdTrackName1] = { name = "track_name 1", controlID = kIdTrackName1, faderBankTrack = 0, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/0/name' },
    -- it seems required and sufficient to provide the fields for the track number in the assignments table to display the track number correctly in the mixer, for completeness we leave them in the items table as well
    [kIdTrackNumber1] = { name = "track_number 1", controlID = kIdTrackNumber1, faderBankTrack = 0, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/0/number' },
    [kIdVolume1] = { name = 'volume 1', valueMode = kAssignScaled, faderBankTrack = 0, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/0/volume', OSCTouched = '/mas/tracks/0/volume/z', OSCValueString = '/mas/tracks/0/level' },
    [kIdSelect1] = { name = 'select 1', faderBankTrack = 0, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/0/select', OSCTouched = '/mas/tracks/0/select/z' },
    [kIdMute1] = { name = 'mute 1', faderBankTrack = 0, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/0/mute', OSCTouched = '/mas/tracks/0/mute/z' },
    [kIdSolo1] = { name = 'solo 1', faderBankTrack = 0, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/0/solo', OSCTouched = '/mas/tracks/0/solo/z' },
    [kIdPan1] = { name = 'pan 1', valueMode = kAssignScaled, faderBankTrack = 0, trackParam = AUPAN, OSCValueChange = '/mas/tracks/0/pan', OSCTouched = '/mas/tracks/0/pan/z' },
    [kIdAutomation1] = { name = 'automation 1', faderBankTrack = 0, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/0/automation', OSCTouched = '/mas/tracks/0/automation/z' },
    [kIdLevelL1] = { name = 'level-l 1', faderBankTrack = 0, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/0/level-l', OSCTouched = '/mas/tracks/0/level-l/z' },
    [kIdLevelR1] = { name = 'level-r 1', faderBankTrack = 0, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/0/level-r', OSCTouched = '/mas/tracks/0/level-r/z' },

    --- Track 2
    [kIdTrackName2] = { name = "track_name 2", controlID = kIdTrackName2, faderBankTrack = 1, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/1/name' },
    [kIdTrackNumber2] = { name = "track_number 2", controlID = kIdTrackNumber2, faderBankTrack = 1, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/1/number' },
    [kIdVolume2] = { name = 'volume 2', valueMode = kAssignScaled, faderBankTrack = 1, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/1/volume', OSCTouched = '/mas/tracks/1/volume/z', OSCValueString = '/mas/tracks/1/level' },
    [kIdSelect2] = { name = 'select 2', faderBankTrack = 1, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/1/select', OSCTouched = '/mas/tracks/1/select/z' },
    [kIdMute2] = { name = 'mute 2', faderBankTrack = 1, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/1/mute', OSCTouched = '/mas/tracks/1/mute/z' },
    [kIdSolo2] = { name = 'solo 2', faderBankTrack = 1, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/1/solo', OSCTouched = '/mas/tracks/1/solo/z' },
    [kIdPan2] = { name = 'pan 2', valueMode = kAssignScaled, faderBankTrack = 1, trackParam = AUPAN, OSCValueChange = '/mas/tracks/1/pan', OSCTouched = '/mas/tracks/1/pan/z' },
    [kIdAutomation2] = { name = 'automation 2', faderBankTrack = 1, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/1/automation', OSCTouched = '/mas/tracks/1/automation/z' },
    [kIdLevelL2] = { name = 'level-l 2', faderBankTrack = 1, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/1/level-l', OSCTouched = '/mas/tracks/1/level-l/z' },
    [kIdLevelR2] = { name = 'level-r 2', faderBankTrack = 1, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/1/level-r', OSCTouched = '/mas/tracks/1/level-r/z' },

    --- Track 3
    [kIdTrackName3] = { name = "track_name 3", controlID = kIdTrackName3, faderBankTrack = 2, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/2/name' },
    [kIdTrackNumber3] = { name = "track_number 3", controlID = kIdTrackNumber3, faderBankTrack = 2, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/2/number' },
    [kIdVolume3] = { name = 'volume 3', valueMode = kAssignScaled, faderBankTrack = 2, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/2/volume', OSCTouched = '/mas/tracks/2/volume/z', OSCValueString = '/mas/tracks/2/level' },
    [kIdSelect3] = { name = 'select 3', faderBankTrack = 2, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/2/select', OSCTouched = '/mas/tracks/2/select/z' },
    [kIdMute3] = { name = 'mute 3', faderBankTrack = 2, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/2/mute', OSCTouched = '/mas/tracks/2/mute/z' },
    [kIdSolo3] = { name = 'solo 3', faderBankTrack = 2, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/2/solo', OSCTouched = '/mas/tracks/2/solo/z' },
    [kIdPan3] = { name = 'pan 3', valueMode = kAssignScaled, faderBankTrack = 2, trackParam = AUPAN, OSCValueChange = '/mas/tracks/2/pan', OSCTouched = '/mas/tracks/2/pan/z' },
    [kIdAutomation3] = { name = 'automation 3', faderBankTrack = 2, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/2/automation', OSCTouched = '/mas/tracks/2/automation/z' },
    [kIdLevelL3] = { name = 'level-l 3', faderBankTrack = 2, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/2/level-l', OSCTouched = '/mas/tracks/2/level-l/z' },
    [kIdLevelR3] = { name = 'level-r 3', faderBankTrack = 2, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/2/level-r', OSCTouched = '/mas/tracks/2/level-r/z' },

    --- Track 4
    [kIdTrackName4] = { name = "track_name 4", controlID = kIdTrackName4, faderBankTrack = 3, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/3/name' },
    [kIdTrackNumber4] = { name = "track_number 4", controlID = kIdTrackNumber4, faderBankTrack = 3, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/3/number' },
    [kIdVolume4] = { name = 'volume 4', valueMode = kAssignScaled, faderBankTrack = 3, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/3/volume', OSCTouched = '/mas/tracks/3/volume/z', OSCValueString = '/mas/tracks/3/level' },
    [kIdSelect4] = { name = 'select 4', faderBankTrack = 3, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/3/select', OSCTouched = '/mas/tracks/3/select/z' },
    [kIdMute4] = { name = 'mute 4', faderBankTrack = 3, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/3/mute', OSCTouched = '/mas/tracks/3/mute/z' },
    [kIdSolo4] = { name = 'solo 4', faderBankTrack = 3, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/3/solo', OSCTouched = '/mas/tracks/3/solo/z' },
    [kIdPan4] = { name = 'pan 4', valueMode = kAssignScaled, faderBankTrack = 3, trackParam = AUPAN, OSCValueChange = '/mas/tracks/3/pan', OSCTouched = '/mas/tracks/3/pan/z' },
    [kIdAutomation4] = { name = 'automation 4', faderBankTrack = 3, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/3/automation', OSCTouched = '/mas/tracks/3/automation/z' },
    [kIdLevelL4] = { name = 'level-l 4', faderBankTrack = 3, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/3/level-l', OSCTouched = '/mas/tracks/3/level-l/z' },
    [kIdLevelR4] = { name = 'level-r 4', faderBankTrack = 3, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/3/level-r', OSCTouched = '/mas/tracks/3/level-r/z' },

    --- Track 5
    [kIdTrackName5] = { name = "track_name 5", controlID = kIdTrackName5, faderBankTrack = 4, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/4/name' },
    [kIdTrackNumber5] = { name = "track_number 5", controlID = kIdTrackNumber5, faderBankTrack = 4, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/4/number' },
    [kIdVolume5] = { name = 'volume 5', valueMode = kAssignScaled, faderBankTrack = 4, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/4/volume', OSCTouched = '/mas/tracks/4/volume/z', OSCValueString = '/mas/tracks/4/level' },
    [kIdSelect5] = { name = 'select 5', faderBankTrack = 4, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/4/select', OSCTouched = '/mas/tracks/4/select/z' },
    [kIdMute5] = { name = 'mute 5', faderBankTrack = 4, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/4/mute', OSCTouched = '/mas/tracks/4/mute/z' },
    [kIdSolo5] = { name = 'solo 5', faderBankTrack = 4, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/4/solo', OSCTouched = '/mas/tracks/4/solo/z' },
    [kIdPan5] = { name = 'pan 5', valueMode = kAssignScaled, faderBankTrack = 4, trackParam = AUPAN, OSCValueChange = '/mas/tracks/4/pan', OSCTouched = '/mas/tracks/4/pan/z' },
    [kIdAutomation5] = { name = 'automation 5', faderBankTrack = 4, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/4/automation', OSCTouched = '/mas/tracks/4/automation/z' },
    [kIdLevelL5] = { name = 'level-l 5', faderBankTrack = 4, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/4/level-l', OSCTouched = '/mas/tracks/4/level-l/z' },
    [kIdLevelR5] = { name = 'level-r 5', faderBankTrack = 4, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/4/level-r', OSCTouched = '/mas/tracks/4/level-r/z' },

    --- Track 6
    [kIdTrackName6] = { name = "track_name 6", controlID = kIdTrackName6, faderBankTrack = 5, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/5/name' },
    [kIdTrackNumber6] = { name = "track_number 6", controlID = kIdTrackNumber6, faderBankTrack = 5, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/5/number' },
    [kIdVolume6] = { name = 'volume 6', valueMode = kAssignScaled, faderBankTrack = 5, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/5/volume', OSCTouched = '/mas/tracks/5/volume/z', OSCValueString = '/mas/tracks/5/level' },
    [kIdSelect6] = { name = 'select 6', faderBankTrack = 5, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/5/select', OSCTouched = '/mas/tracks/5/select/z' },
    [kIdMute6] = { name = 'mute 6', faderBankTrack = 5, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/5/mute', OSCTouched = '/mas/tracks/5/mute/z' },
    [kIdSolo6] = { name = 'solo 6', faderBankTrack = 5, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/5/solo', OSCTouched = '/mas/tracks/5/solo/z' },
    [kIdPan6] = { name = 'pan 6', valueMode = kAssignScaled, faderBankTrack = 5, trackParam = AUPAN, OSCValueChange = '/mas/tracks/5/pan', OSCTouched = '/mas/tracks/5/pan/z' },
    [kIdAutomation6] = { name = 'automation 6', faderBankTrack = 5, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/5/automation', OSCTouched = '/mas/tracks/5/automation/z' },
    [kIdLevelL6] = { name = 'level-l 6', faderBankTrack = 5, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/5/level-l', OSCTouched = '/mas/tracks/5/level-l/z' },
    [kIdLevelR6] = { name = 'level-r 6', faderBankTrack = 5, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/5/level-r', OSCTouched = '/mas/tracks/5/level-r/z' },

    --- Track 7
    [kIdTrackName7] = { name = "track_name 7", controlID = kIdTrackName7, faderBankTrack = 6, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/6/name' },
    [kIdTrackNumber7] = { name = "track_number 7", controlID = kIdTrackNumber7, faderBankTrack = 6, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/6/number' },
    [kIdVolume7] = { name = 'volume 7', valueMode = kAssignScaled, faderBankTrack = 6, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/6/volume', OSCTouched = '/mas/tracks/6/volume/z', OSCValueString = '/mas/tracks/6/level' },
    [kIdLevelL7] = { name = 'level-l 7', faderBankTrack = 6, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/6/level-l', OSCTouched = '/mas/tracks/6/level-l/z' },
    [kIdMute7] = { name = 'mute 7', faderBankTrack = 6, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/6/mute', OSCTouched = '/mas/tracks/6/mute/z' },
    [kIdSolo7] = { name = 'solo 7', faderBankTrack = 6, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/6/solo', OSCTouched = '/mas/tracks/6/solo/z' },
    [kIdPan7] = { name = 'pan 7', valueMode = kAssignScaled, faderBankTrack = 6, trackParam = AUPAN, OSCValueChange = '/mas/tracks/6/pan', OSCTouched = '/mas/tracks/6/pan/z' },
    [kIdAutomation7] = { name = 'automation 7', faderBankTrack = 6, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/6/automation', OSCTouched = '/mas/tracks/6/automation/z' },
    [kIdLevelR7] = { name = 'level-r 7', faderBankTrack = 6, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/6/level-r', OSCTouched = '/mas/tracks/6/level-r/z' },
    [kIdSelect7] = { name = 'select 7', faderBankTrack = 6, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/6/select', OSCTouched = '/mas/tracks/6/select/z' },

    --- Track 8
    [kIdTrackName8] = { name = "track_name 8", controlID = kIdTrackName8, faderBankTrack = 7, trackParam = CS_NAME, shortParamName = '@tn', OSCValueString = '/mas/tracks/7/name' },
    [kIdTrackNumber8] = { name = "track_number 8", controlID = kIdTrackNumber8, faderBankTrack = 7, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/7/number' },
    [kIdVolume8] = { name = 'volume 8', valueMode = kAssignScaled, faderBankTrack = 7, trackParam = AUVOLUME, OSCValueChange = '/mas/tracks/7/volume', OSCTouched = '/mas/tracks/7/volume/z', OSCValueString = '/mas/tracks/7/level' },
    [kIdSelect8] = { name = 'select 8', faderBankTrack = 7, trackParam = CS_SELECT, OSCValueChange = '/mas/tracks/7/select', OSCTouched = '/mas/tracks/7/select/z' },
    [kIdMute8] = { name = 'mute 8', faderBankTrack = 7, trackParam = CS_MUTE, OSCValueChange = '/mas/tracks/7/mute', OSCTouched = '/mas/tracks/7/mute/z' },
    [kIdSolo8] = { name = 'solo 8', faderBankTrack = 7, trackParam = CS_SOLO, OSCValueChange = '/mas/tracks/7/solo', OSCTouched = '/mas/tracks/7/solo/z' },
    [kIdPan8] = { name = 'pan 8', valueMode = kAssignScaled, faderBankTrack = 7, trackParam = AUPAN, OSCValueChange = '/mas/tracks/7/pan', OSCTouched = '/mas/tracks/7/pan/z' },
    [kIdAutomation8] = { name = 'automation 8', faderBankTrack = 7, trackParam = CS_AUTO, OSCValueChange = '/mas/tracks/7/automation', OSCTouched = '/mas/tracks/7/automation/z' },
    [kIdLevelL8] = { name = 'level-l 8', faderBankTrack = 7, trackParam = CS_PEAKHOLD0, OSCValueChange = '/mas/tracks/7/level-l', OSCTouched = '/mas/tracks/7/level-l/z' },
    [kIdLevelR8] = { name = 'level-r 8', faderBankTrack = 7, trackParam = CS_PEAKHOLD0, paramOffset = 1, OSCValueChange = '/mas/tracks/7/level-r', OSCTouched = '/mas/tracks/7/level-r/z' }
}

--------------------------------------------

function controller_info()
    return {
        model = 'MASCHINE MK3 OSC',
        manufacturer = 'Native Instruments',
        version = 1,
        copyright = [[Copyright © 2021 Native Instruments GmbH. All rights reserved.]],
        OSC_pattern = 'MASCHINE MK3',

        items = controls,

        -- The root zone "Control Surface: MASCHINE MK3" seems to be added automatically by Logic
        assignments = {

            --------------------------------------------------------------
            { zone = 'Transport' },
            ----------------------------
            { control = 'play' },
            { control = 'stop' },
            { control = 'pause' },
            { control = 'record' },
            { control = 'forward' },
            { control = 'rewind' },
            { control = 'cycle' },

            { control = 'forward_shuttle' },
            { control = 'rewind_shuttle' },
            { control = 'tap_tempo' },
            { control = 'quantize' },
            { control = 'restart' },

            --------------------------------------------------------------
            { zone = 'Control' },
            ----------------------------
            { control = 'undo' },
            { control = 'redo' },
            { control = 'metronome' },
            { control = 'count_in' },
            { control = 'move_cycle_forward' },
            { control = 'move_cycle_backwards' },

            --------------------------------------------------------------
            { zone = 'Navigation' },
            ----------------------------
            { control = 'prev_track' },
            { control = 'next_track' },
            { control = 'prev_object' },
            { control = 'next_object' },

            { control = 'fader-bank-' },
            { control = 'fader-bank+' },

			{ control = 'current_track', valueMode = kAssignDirect, CSTrack = true, trackParam = CS_NAME, paramName = '@t#', shortParamName = '@t#', OSCLabel = '/mas/tracks/current_index', OSCValueString = '/mas/tracks/current_name' },
            { control = 'parameter0' },
            { control = 'volume-master' },

            --- Track 1
            { control = 'track_name 1' },
            -- the assignments table seems to require these fields here to correctly show track number
            { control = 'track_number 1', faderBankTrack = 0, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/0/number' },
            { control = 'volume 1' },
            { control = 'select 1' },
            { control = 'mute 1' },
            { control = 'solo 1' },
            { control = 'pan 1' },
            { control = 'automation 1' },
            { control = 'level-l 1' },
            { control = 'level-r 1' },

            --- Track 2
            { control = 'track_name 2' },
            { control = 'track_number 2', faderBankTrack = 1, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/1/number' },
            { control = 'volume 2' },
            { control = 'select 2' },
            { control = 'mute 2' },
            { control = 'solo 2' },
            { control = 'pan 2' },
            { control = 'automation 2' },
            { control = 'level-l 2' },
            { control = 'level-r 2' },

            --- Track 3
            { control = 'track_name 3' },
            { control = 'track_number 3', faderBankTrack = 2, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/2/number' },
            { control = 'volume 3' },
            { control = 'select 3' },
            { control = 'mute 3' },
            { control = 'solo 3' },
            { control = 'pan 3' },
            { control = 'automation 3' },
            { control = 'level-l 3' },
            { control = 'level-r 3' },

            --- Track 4
            { control = 'track_name 4' },
            { control = 'track_number 4', faderBankTrack = 3, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/3/number' },
            { control = 'volume 4' },
            { control = 'select 4' },
            { control = 'mute 4' },
            { control = 'solo 4' },
            { control = 'pan 4' },
            { control = 'automation 4' },
            { control = 'level-l 4' },
            { control = 'level-r 4' },

            --- Track 5
            { control = 'track_name 5' },
            { control = 'track_number 5', faderBankTrack = 4, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/4/number' },
            { control = 'volume 5' },
            { control = 'select 5' },
            { control = 'mute 5' },
            { control = 'solo 5' },
            { control = 'pan 5' },
            { control = 'automation 5' },
            { control = 'level-l 5' },
            { control = 'level-r 5' },

            --- Track 6
            { control = 'track_name 6' },
            { control = 'track_number 6', faderBankTrack = 5, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/5/number' },
            { control = 'volume 6' },
            { control = 'select 6' },
            { control = 'mute 6' },
            { control = 'solo 6' },
            { control = 'pan 6' },
            { control = 'automation 6' },
            { control = 'level-l 6' },
            { control = 'level-r 6' },

            --- Track 7
            { control = 'track_name 7' },
            { control = 'track_number 7', faderBankTrack = 6, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/6/number' },
            { control = 'volume 7' },
            { control = 'select 7' },
            { control = 'mute 7' },
            { control = 'solo 7' },
            { control = 'pan 7' },
            { control = 'automation 7' },
            { control = 'level-l 7' },
            { control = 'level-r 7' },

            --- Track 8
            { control = 'track_name 8' },
            { control = 'track_number 8', faderBankTrack = 7, trackParam = CS_NAME, shortParamName = '@t#', OSCLabel = '/mas/tracks/7/number' },
            { control = 'volume 8' },
            { control = 'select 8' },
            { control = 'mute 8' },
            { control = 'solo 8' },
            { control = 'pan 8' },
            { control = 'automation 8' },
            { control = 'level-l 8' },
            { control = 'level-r 8' }
        }
    }
end

function CSFeedback()
    return 32 -- maximal length of text feedback
end

function CSLabelSize()
    return 32 -- maximum length of a text label
end